#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""LCM character code converter"""

import os
from unicode_lcm_code import LCM_CODE_LIST



def is_ascii(chraracter):
    """Return true if specified characters is ASCII."""

    return (ord(chraracter) < 128)


def unicode2lcmcode(chraracter):
    """chraracter must be an unicode character"""

    code_str = LCM_CODE_LIST.get(ord(chraracter))
    if code_str:
        chr_lcm_code = int(code_str, 16)
    else:
        chr_lcm_code = 0xA148  # default (0xA148 -> "？")
    return chr_lcm_code

def get_lcm_code_list(string):
    """Get LCM character code list correspinding specified string"""

    # LCMの仕様上LCM文字の左半分がASCIIの場合は右半分はASCII
    # 左半分がASCII隣が全角の場合は右半分はASCIIスペースにする
    # 例えば"aあ"が指定された場合はLCM上は"a あ"となる

    lcm_code_list = []
    strlist = list(string)
    idx = -1
    while(True):

        idx += 1
        if idx == len(strlist):
            break

        if not is_ascii(strlist[idx]):
            lcm_code_list.append(unicode2lcmcode(strlist[idx]))
            continue

        ascii_2byte_code = ord(strlist[idx]) << 8

        if (idx + 1) == len(strlist):
            lcm_code_list.append(ascii_2byte_code)
            break

        if is_ascii(strlist[idx + 1]):
            ascii_2byte_code += ord(strlist[idx + 1])
            idx += 1

        lcm_code_list.append(ascii_2byte_code)

    return lcm_code_list
