#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""PAL command for Crond."""
import os

import libpal.palcmd as palcmd
from pal_info import (ReqTypes, Results)

import pal_config as config

import libpal.crond.ctrl as ctrl

CROND_NAME = "crond"
TARGET_NAME = "target"


class PalCrond(palcmd.PalCmd):
    """Pal Crond command implementation class."""

    def __check_root_node_name(self, entry, name):
        """Root node name check."""

        if entry.get().get_node_name() != name:
            raise InvalidArgError(
                "root_node_name = " + entry.get().get_node_name())

    def __add_part_file(self, entry):
        """Add crontab part file."""

        target = entry.get().get_node(TARGET_NAME).get_text()

        ctrl_ins = getattr(ctrl, target)()
        ctrl_ins.add_part_file(entry.get())

        return

    def __delete_part_file(self, entry):
        """Create crontab part file."""

        target = entry.get().get_node(TARGET_NAME).get_text()

        ctrl_ins = getattr(ctrl, target)()
        ctrl_ins.delete_part_file(entry.get())

        return

    def __add_handler(self, entry):
        """Add Handler."""

        try:
            self.__check_root_node_name(entry, CROND_NAME)

            self.__add_part_file(entry)
            #FIXME ファイル連結処理　ライブラリ使用予
            entry.set_result(Results.OK)

        except (IOError, OSError):
            entry.set_result(Results.ADD_ERROR)
            self.exception("Apply fail!!")

        return entry

    def __delete_handler(self, entry):
        """Delete Handler."""

        try:
            self.__check_root_node_name(entry, CROND_NAME)

            self.__delete_part_file(entry)
            #FIXME ファイル連結処理　ライブラリ使用予
            entry.set_result(Results.OK)

        except (IOError, OSError):
            entry.set_result(Results.DELETE_ERROR)
            self.exception("Delete fail!!")

        return entry

    def __init__(self, option=None):
        """Constructor."""

        palcmd.PalCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.ADD, self.__add_handler)
        self.set_handler(ReqTypes.DELETE, self.__delete_handler)

#
# Main
#

if __name__ == "__main__":
    PalCrond(option=palcmd.PalCmdOption()).handle_request()
