#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""PAL command for setting lump regist"""

from __future__ import with_statement

import os
import shutil

import ionas.nascmd as nascmd
import libpal.palcmd as palcmd
from albg_info import ReqTypes, Results
import albg_info
import pal_config as config

COMPLETE_STATUS = "COMPLETE"

STATUS_PATH = "status"

STATUS_FILE_PATH = config.LUMPREGIST_STATUS_DIR + "/lumpregist"


class PalLumpRegist(palcmd.PalCmd):
    """PAL Lump Regist command implementation class."""

    def __get_file_info(self):

        file_path = STATUS_FILE_PATH
        with open(file_path, "rb") as f:
            info_list = f.read().split(":", 1)
            if len(info_list) == 2:
                status, result = info_list
            else:
                status = info_list[0]
                result = ""

        timestamp = str(int(os.stat(file_path).st_mtime))

        return (status, result, timestamp)

    def __set_status(self, status):

        tmp_file_path = os.path.join(
            config.TEMP_DIR, "lumpregist." + str(os.getpid()))

        with open(tmp_file_path, "wb") as tmp_file:
            tmp_file.write(status)

        shutil.move(tmp_file_path, STATUS_FILE_PATH)

    def __set_status_to_xml(self, entry):

        status, result, timestamp = self.__get_file_info()

        node = entry.get()
        node.append_child("status", status)

        if status == COMPLETE_STATUS:
            node.append_child("result", result)
            node.append_child("timestamp", timestamp)

    def __set_xml_to_status(self, entry):

        self.__set_status(entry.get().get_value(STATUS_PATH))

    def __query_handler(self, entry):

        try:
            self.__set_status_to_xml(entry)
            entry.set_result(Results.OK)

        except:
            self.exception("Query Error.")
            entry.set_result(Results.QUERY_ERROR)

        return entry

    def __apply_handler(self, entry):

        try:
            self.__set_xml_to_status(entry)
            entry.set_result(Results.OK)

        except:
            self.exception("Apply Error.")
            entry.set_result(Results.APPLY_ERROR)

        return entry
 
    def __init__(self, option=None):
        """Constructor."""

        palcmd.PalCmd.__init__(self, option=option)

        lump_regist_handler = lambda entry: self.send_entry(entry,
                        albg_info.ALBG_LUMPREGIST_PATH)

        self.set_handler(ReqTypes.LUMP_REGIST, lump_regist_handler)
        self.set_handler(ReqTypes.QUERY, self.__query_handler)
        self.set_handler(ReqTypes.APPLY, self.__apply_handler)

#
# Main
#

if __name__ == "__main__":
    PalLumpRegist(option=palcmd.PalCmdOption()).handle_request()
