#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""Event handler to initialize environment for RAPS."""

import os
import stat

import ionas.request as request
import evh_config as config
import libevh.evhcmd as evhcmd
from fal_info import (Results, ReqTypes)

from pkgaddon.raps_config import (VAR_RAPS_DIR,)


EVENTINFO_NAME = "eventinfo"
VOLUME_NAME_PATH = "volume/name"
CATEGORY_ATTR = "category"
ACTION_ATTR = "action"
PREVIOUS_ATTR = "previous"

VOLUME_CATEGORY = "volume"
MOUNT_ACTION = "mount"


class EvhRapsInit(evhcmd.EvhCmd):
    """Event handler to initialize environment for RAPS."""

    def __handle_volume_event(self, eventinfo, entry):
        """Handle volume event."""

        action = eventinfo.get_attr_value(ACTION_ATTR)
        is_previous = eventinfo.is_set_to_true(PREVIOUS_ATTR)

        if (eventinfo.get_node(VOLUME_NAME_PATH).get_text() !=
                config.INTERNAL_VOL):

            # Non target events
            entry.set_result(Results.OK)
            return entry

        if action == MOUNT_ACTION:
            if not is_previous and not os.path.exists(VAR_RAPS_DIR):
                os.makedirs(VAR_RAPS_DIR)
                os.chmod(VAR_RAPS_DIR,
                            stat.S_IRWXU | stat.S_IRWXG | stat.S_IRWXO)

        entry.set_result(Results.OK)
        return entry

    def __event_handler(self, entry):
        """Handle "event" request."""

        try:
            eventinfo = entry.get().get_node(EVENTINFO_NAME)
            if eventinfo.get_attr_value(CATEGORY_ATTR) == VOLUME_CATEGORY:
                return self.__handle_volume_event(eventinfo, entry)

        except:
            # Non target events
            pass

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)


#
# Main
#

if __name__ == "__main__":
    EvhRapsInit(option=evhcmd.EvhCmdOption()).handle_request()
