# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.conf import settings
from django.views.generic import View
from django.views.decorators.csrf import csrf_exempt
from django.utils.decorators import method_decorator
from django import forms
from django.shortcuts import render_to_response
from iobase.exception import NotFoundError
from access import readable, writable
import os
import datetime
import time
import random
import pwd


CURRENT_DIR = os.path.dirname(os.path.abspath(__file__))
TEMPLATES_DIR = os.path.join(CURRENT_DIR, 'templates')


def _get_locale():
    return 'ja'  # FIXME


def _make_token():
    return ''.join([random.choice('abcdefghijklmnopqrstuvwxyz0123456789') for i in range(50)])


def _temporary_access_is_enabled():
    try:
        from ionas.request import Request, Entry, EntryList
        from product_conf import SUDO_CMD
        from fal_info import FAL_REMOTELINK3_PATH as command
        entry = Entry(u'load', u"""<remotelink3/>""")
        request = Request(command, filter_cmd=SUDO_CMD)
        response = request.request(entry)
        entry = response.get_entrylist().get_entry()
        return entry.get().get_value(u'remoteaccess/timelimit/@enable', 'false') == 'true'
    except:
        return False


#def _check_sessionid(sessionid):
def _get_username(sessionid):
    username = None
    try:
        from ionas.request import Request, Entry, EntryList
        from product_conf import SUDO_CMD
        from fal_info import FAL_REMOTEACCESS_PATH as command
        entry = Entry(u'check_sessionid', u"""<remoteaccess><session><id/></session></remoteaccess>""")
        entry.get().get_node(u'session/id').set_text(sessionid)
        request = Request(command, filter_cmd=SUDO_CMD)
        response = request.request(entry)
        entry = response.get_entrylist().get_entry()
        username = entry.get().get_value(u'session/username', None)
    except:
        username = None
    return username


def _encode_base64(path):
    from base64 import b64encode
    encoded_data = ''
    with open(path, 'rb') as f:
        encoded_data = b64encode(f.read())
    return encoded_data


def _get_entry(username, pathlist):
    try:
        from ionas.request import Request, Entry, EntryList
        from product_conf import SUDO_CMD
        from fal_info import FAL_REMOTELINK3_PATH as command
        from fal_info import Results
        entry = Entry(u'get_timelimit', u"""<remotelink3><timelimit><pathlist/><registerinfo/></timelimit></remotelink3>""")
        registerinfo_node = entry.get().get_node(u'timelimit/registerinfo')
        registerinfo_node.append_child(u'username', text=username)
        pathlist_node = entry.get().get_node(u'timelimit/pathlist')
        for path in pathlist:
            pathlist_node.append_child(u'path', text=path)
        request = Request(command, filter_cmd=SUDO_CMD)
        response = request.request(entry)
        entry = response.get_entrylist().get_entry()
        if entry.get_result() in [Results.OK]:
            return {
                'pincode': entry.get().get_value(u'timelimit/pincode', None),
                #'expire': int(entry.get().get_value(u'timelimit/expire/time', '0')),
                'expire': datetime.datetime(*time.localtime(int(entry.get().get_value(u'timelimit/expire/time', '0')))[:6]),
                'client_max_num': int(entry.get().get_value(u'timelimit/clientmax', '0')),
                'access_max_num': int(entry.get().get_value(u'timelimit/accessmax', '0')),
            }
        else:
            return None
    except:
        return None


def _add_pincode(pathlist, expire, client_max_num, access_max_num, username):
    try:
        from ionas.request import Request, Entry, EntryList
        from product_conf import SUDO_CMD
        from fal_info import FAL_REMOTELINK3_PATH, Results
        # make pincode
        entry = Entry('add_timelimit', """<remotelink3>
            <timelimit>
                <pathlist/>
                <expire>
                    <time/>
                </expire>
                <clientmax/>
                <accessmax/>
                <registerinfo>
                    <username/>
                </registerinfo>
            </timelimit>
        </remotelink3>""")
        remotelink3_node = entry.get()
        pathlist_node = remotelink3_node.get_node('timelimit/pathlist')
        for path in pathlist:
            pathlist_node.append_child('path', text=path)
        epoch = int(time.mktime(datetime.datetime.combine(expire, datetime.time(23, 59, 59)).timetuple()))
        remotelink3_node.get_node('timelimit/expire/time').set_text(str(epoch or 0))
        remotelink3_node.get_node('timelimit/clientmax').set_text(str(client_max_num or 0))
        remotelink3_node.get_node('timelimit/accessmax').set_text(str(access_max_num or 0))
        remotelink3_node.get_node('timelimit/registerinfo/username').set_text(username)
        req = Request(FAL_REMOTELINK3_PATH, filter_cmd=SUDO_CMD)
        res = req.request(entry)
        entry = res.get_entrylist().get_entry()

        # return entry.get().get_value('timelimit/pincode')
        _pincode, _errmsg = None, '生成に失敗しました'
        if entry.get_result() in [Results.OK]:
            _pincode = entry.get().get_value('timelimit/pincode')
            _errmsg = None
        else:
            pass
        return _pincode, _errmsg
    except:
        return None, '生成に失敗しました'


def _mod_pincode(pincode, pathlist, expire, client_max_num, access_max_num):
    try:
        from ionas.request import Request, Entry, EntryList
        from product_conf import SUDO_CMD
        from fal_info import FAL_REMOTELINK3_PATH as command
        from fal_info import Results
        # mod pincode
        entry = Entry('modify_timelimit', """<remotelink3>
            <timelimit>
                <pincode/>
                <pathlist/>
                <expire>
                    <time/>
                </expire>
                <clientmax/>
                <accessmax/>
            </timelimit>
        </remotelink3>""")
        remotelink3_node = entry.get()
        pathlist_node = remotelink3_node.get_node('timelimit/pathlist')
        for path in pathlist:
            pathlist_node.append_child('path', text=path)
        epoch = int(time.mktime(datetime.datetime.combine(expire, datetime.time(23, 59, 59)).timetuple()))
        remotelink3_node.get_node('timelimit/expire/time').set_text(str(epoch or 0))
        remotelink3_node.get_node('timelimit/clientmax').set_text(str(client_max_num or 0))
        remotelink3_node.get_node('timelimit/accessmax').set_text(str(access_max_num or 0))
        remotelink3_node.get_node('timelimit/pincode').set_text(pincode)
        req = Request(command, filter_cmd=SUDO_CMD)
        res = req.request(entry)
        entry = res.get_entrylist().get_entry()
        
        _pincode, _errmsg = pincode, '変更に失敗しました'
        if entry.get_result() in [Results.OK]:
            _pincode = entry.get().get_value('timelimit/pincode')
            _errmsg = None
        else:
            pass
        return _pincode, _errmsg
    except:
        return pincode, '変更に失敗しました'


def _make_qrcode(pincode):
    try:
        from ionas.request import Request, Entry, EntryList
        from product_conf import SUDO_CMD
        from fal_info import FAL_REMOTELINK3_PATH, Results
        # make qrcode
        directory = settings.MEDIA_ROOT
        entry = Entry('create_qrcode', """<remotelink3><qrcode/></remotelink3>""")
        remotelink3_node = entry.get()
        remotelink3_node.get_node('qrcode').append_child('directory', text=directory)
        remotelink3_node.get_node('qrcode').append_child('pincode', text=pincode)
        req = Request(FAL_REMOTELINK3_PATH, filter_cmd=SUDO_CMD)
        res = req.request(entry)
        entry = res.get_entrylist().get_entry()
        filename = entry.get().get_value('qrcode/filename')
        # encode base64
        qrcode = _encode_base64(os.path.join(directory, filename))
        return qrcode
    except:
        return None


def _gettext(name, args={}):
    try:
        from ionas.request import Request, Entry, EntryList
        from product_conf import SUDO_CMD
        from fal_info import Results
        from fal_info import FAL_MESSAGES_PATH as falcmd
        from fal_msginfo import Msg as Message

        code = getattr(Message, name)

        entry = Entry('query', """<messages><messageinfo><arglist/></messageinfo></messages>""")
        messageinfo_node = entry.get().get_node('messageinfo')
        messageinfo_node.append_child('code', text=code)
        arglist_node = messageinfo_node.get_node('arglist')
        for k, v in args.iteritems():
            arglist_node.append_child('arg', attrs={'name': k}, text=str(v))

        req = Request(falcmd, filter_cmd=SUDO_CMD)
        res = req.request(entry)

        entry = res.get_entrylist().get_entry()
        message = entry.get().get_value('messageinfo/message', '')
        return message
    except:
        return ''


def _logging(name, args={}):
    try:
        from ionas.request import Request, Entry, EntryList
        from product_conf import SUDO_CMD
        from fal_info import Results
        from fal_info import FAL_EVENTLOG_PATH as falcmd
        from fal_msginfo import Msg as Message

        code = getattr(Message, name)

        entry = Entry('add', """<eventlog><log><arglist/></log></eventlog>""")
        log_node = entry.get().get_node('log')
        log_node.append_child('code', text=code)
        arglist_node = log_node.get_node('arglist')
        for k, v in args.iteritems():
            arglist_node.append_child('arg', attrs={'name': k}, text=str(v))

        req = Request(falcmd, filter_cmd=SUDO_CMD)
        res = req.request(entry)

        entry = res.get_entrylist().get_entry()
        return entry.get_result() in [Results.OK]
    except:
        return False


def _is_exist_node(node, path):
    """Check whether path exists in specified node"""
    try:
        node.get_node(path)
    except NotFoundError:
        return False
    return True


def _path_to_share_name(path):
    return path.strip("/").split("/")[0]


def _followall(toppath):
    for root, dirs, files in os.walk(toppath, topdown=True):
        yield root
        for name in files:
            yield os.path.join(root, name)


def _get_shares_info():
    info = []
    try:
        from ionas.request import Request, Entry, EntryList
        from product_conf import SUDO_CMD
        from fal_info import FAL_SHARES_PATH as command
        entry = Entry(u'load')

        request = Request(command, filter_cmd=SUDO_CMD)
        response = request.request(entry)
        result = response.get_entrylist().get_entry().get()

        for share_node in result.get_node_list('sharelist/share'):
            acl_enabled = False
            if _is_exist_node(share_node, './permission/acl'):
                acl_enabled = \
                    share_node.get_node('permission/acl').is_enabled()

            info_dic = {
                'name': share_node.get_value('name'),
                'path': share_node.get_value('path'),
                'acl_enabled': acl_enabled,
            }
            info.append(info_dic)
    except:
        pass

    return info


def _is_acl_accesible_path(full_path, username):
    uid = pwd.getpwnam(username).pw_uid
    if os.path.isdir(full_path.encode('utf_8')):
        paths = _followall(full_path.encode('utf_8'))
        for path in paths:
            if not readable(uid, path, None):
                return False
    else:
        if not readable(uid, full_path, 'utf_8'):
            return False
    return True


def _is_acl_accesible_pathlist(pathlist, username):
    try:
        shares = _get_shares_info()
        for path in pathlist:
            share = [x for x in shares
                     if x['name'] == _path_to_share_name(path)]
            if not share[0]['acl_enabled']:
                continue
            haed, tail = os.path.split(share[0]['path'])
            full_path = os.path.join(haed, path.lstrip("/"))
            if not _is_acl_accesible_path(full_path, username):
                return False
    except:
        return False

    return True


# FIXME
#class TokenForm(forms.Form):
#
#    token = forms.CharField(widget=forms.HiddenInput())


class TemporaryForm0(forms.Form):

    def __init__(self, pathlist, *args, **kwargs):
        super(TemporaryForm0, self).__init__(*args, **kwargs)
        self.fields['pathlist'] = forms.MultipleChoiceField(required=True,
                                                            widget=forms.CheckboxSelectMultiple,
                                                            choices=[(path, path) for path in pathlist],
                                                            initial=pathlist)
        self.fields['session'] = forms.CharField(required=True)


class TemporaryForm1(forms.Form):

    token = forms.CharField(widget=forms.HiddenInput())

    def __init__(self, *args, **kwargs):
        super(TemporaryForm1, self).__init__(*args, **kwargs)
        self.fields['pincode'] = forms.CharField(label='PINコード',
                                                 required=False,
                                                 max_length=1024,
                                                 initial='')
        date_format = '%Y/%m/%d'  # FIXME
        self.fields['expire'] = forms.DateField(label='公開期限',
                                                required=False,
                                                input_formats=[date_format],
                                                widget=forms.DateField.widget(format=date_format),
                                                initial=datetime.date.today)
        self.fields['client_max_num'] = forms.IntegerField(label='端末台数',
                                                           required=False,
                                                           widget=forms.IntegerField.widget(attrs={'maxlength': 3}),
                                                           max_value=999,
                                                           initial=0)
        self.fields['access_max_num'] = forms.IntegerField(label='アクセス回数',
                                                           required=False,
                                                           widget=forms.IntegerField.widget(attrs={'maxlength': 3}),
                                                           max_value=999,
                                                           initial=0)


class TemporaryView(View):

    template_name = os.path.join(TEMPLATES_DIR, 'template.html')

    def post(self, request):
        enabled = _temporary_access_is_enabled()
        form0 = TemporaryForm0(request.POST.getlist('pathlist'), data=request.POST)
        form1 = TemporaryForm1(data=request.POST)
        message = None
        pincode = None
        qrcode = None
        locale = _get_locale()

        if enabled:
            if form0.is_valid():
                # 送られてきたデータを受け取る
                username = _get_username(form0.cleaned_data.get('session', ''))  # 紐付いたユーザー名を取得できれば有効なセッション
                pathlist = form0.cleaned_data.get('pathlist', [])
                # 送られてきたデータに該当する既存のエントリーがないか確認する
                entry_info = _get_entry(username, pathlist)
                if isinstance(entry_info, dict):
                    # 既存のエントリーがあれば変更モードにする
                    # また、そのエントリーのPINコードのQRコードを表示する
                    pincode = entry_info['pincode']
                    qrcode = _make_qrcode(pincode)
                if username:
                    if 'add' in request.POST:
                        if form1.is_valid():
                            if not _is_acl_accesible_pathlist(pathlist, username):
                                message = "アクセス権限がありません接続先の機器の設定をご確認ください"

                            else:
                                if request.session.get('token') == form1.cleaned_data['token']:
                                    pincode, message = _add_pincode(pathlist,
                                                                    form1.cleaned_data.get('expire', datetime.date.today()),
                                                                    form1.cleaned_data.get('client_max_num', 0),
                                                                    form1.cleaned_data.get('access_max_num', 0),
                                                                    username)
                                    if pincode:
                                        qrcode = _make_qrcode(pincode)
                                    if pincode:
                                        #_gettext('MSG_RL3_CREATE_TIMELIMIT_PINCODE')
                                        _logging('MSG_RL3_CREATE_TIMELIMIT_PINCODE')
                                    else:
                                        #_gettext('ERR_RL3_CREATE_TIMELIMIT_PINCODE')
                                        _logging('ERR_RL3_CREATE_TIMELIMIT_PINCODE')
                                else:
                                    # ブラウザのリロード等
                                    pass
                            initial = form1.cleaned_data
                            initial['token'] = request.session['token'] = _make_token()
                            initial['pincode'] = pincode if pincode else ''
                            form1 = TemporaryForm1(initial=initial)
                        else:
                            pass
                    elif 'mod' in request.POST:
                        if form1.is_valid():
                            if not _is_acl_accesible_pathlist(pathlist, username):
                                message = "アクセス権限がありません接続先の機器の設定をご確認ください"
                            else:
                                if request.session.get('token') == form1.cleaned_data['token']:
                                    pincode, message = _mod_pincode(form1.cleaned_data.get('pincode'),
                                                                    pathlist,
                                                                    form1.cleaned_data.get('expire', datetime.date.today()),
                                                                    form1.cleaned_data.get('client_max_num', 0),
                                                                    form1.cleaned_data.get('access_max_num', 0))
                                    if pincode:
                                        qrcode = _make_qrcode(pincode)
                                    if pincode:
                                        #_gettext('MSG_RL3_CHANGE_TIMELIMIT_PINCODE')
                                        _logging('MSG_RL3_CHANGE_TIMELIMIT_PINCODE')
                                    else:
                                        #_gettext('ERR_RL3_CHANGE_TIMELIMIT_PINCODE')
                                        _logging('ERR_RL3_CHANGE_TIMELIMIT_PINCODE')
                                else:
                                    # ブラウザのリロード等
                                    pass
                            initial = form1.cleaned_data
                            initial['token'] = request.session['token'] = _make_token()
                            initial['pincode'] = pincode if pincode else ''
                            form1 = TemporaryForm1(initial=initial)
                        else:
                            pass
                    else:
                        # Remote Link Filesからデータが送られてきた時
                        initial = {}
                        if isinstance(entry_info, dict):
                            initial = entry_info
                        initial['token'] = request.session['token'] = _make_token()
                        form1 = TemporaryForm1(initial=initial)
                else:
                    # セッションが有効でない(そもそもsessionがない場合も含む)
                    token = _make_token()
                    request.session['token'] = token
                    form1 = TemporaryForm1(initial={'token': token})
            else:
                # 送られてきたパラメータが不正
                token = _make_token()
                request.session['token'] = token
                form1 = TemporaryForm1(initial={'token': token})
        else:
            # 機能が無効
            token = _make_token()
            request.session['token'] = token
            form1 = TemporaryForm1(initial={'token': token})

        return render_to_response(self.template_name, {
            'enabled': enabled,
            'form0': form0,
            'form1': form1,
            'message': message,
            'qrcode': qrcode,
            'locale': locale,
        })

    # NOTE: CSRF対策を無効化
    #       RemoteLinkFilesからはPOSTメソッドでcsrf_tokenなしのデータが送られてくるため。
    #       一応sessionidが送られてくるのでそれで検証できるはず。
    @method_decorator(csrf_exempt)
    def dispatch(self, *args, **kwargs):
        return super(TemporaryView, self).dispatch(*args, **kwargs)
