#!/bin/sh
[ -f /etc/beta ] && set -x
OPTION="$1"
PINCODE="$2"

BOOT_PATH='/boot'
PINCODE_INIT_PATH="${BOOT_PATH}/.pincode_init"
INIT_PINCODE_PATH="${BOOT_PATH}/.init_pincode"
PINCODE_PATH="${BOOT_PATH}/.pincode"
MAC_PATH="${BOOT_PATH}/.cryptmac.asc"

GETXMLRESULT='/opt/rl3/bin/getxmlresult.py'

LOCK_PATH='/var/lock/pincode.lock'

REMOUNT_ROOT_WRITE_CMD="/usr/local/bin/remount_root_write"
REMOUNT_ROOT_READ_ONLY_CMD="/usr/local/bin/remount_root_read_only"

__try_lock() {
	COUNT=0
	TIMEOUT=$1
	[ "${TIMEOUT}" = '' ] && TIMEOUT=30
	while [ ${COUNT} -lt ${TIMEOUT} ]; do
		sudo ln -s "$$" "${LOCK_PATH}" && return 0
		COUNT=$((COUNT+1))
		sleep 1
	done
	return 1
}

__unlock() {
	sudo rm "${LOCK_PATH}" || return 1
	return 0
}

__try_lock 30 || sudo echo "$0 lock failed. ignore." >> /dev/console

__exit() {
	local RESULT=$1
	__unlock
	exit ${RESULT}
}

__localhost_macaddr() {
	local MACADDR=`ifconfig eth0 | grep 'HWaddr' | sed 's/.*HWaddr //' | sed 's/[: ]//g'`
	echo -n "${MACADDR}"
}

__pincode_test() {
	local MACADDR=`__localhost_macaddr`
	echo -n "${MACADDR}00000000000000000000"
}

__pincode_regist_init() {
	local PINCODE="$1"
	local KENSA="$2"
	local PINCODE_INIT=`__pincode_init`
	if [ "${PINCODE}" = '' ]; then
		PINCODE="${PINCODE_INIT}"
	fi
	[ "${PINCODE}" = '' ] && return 1
	if [ ! -f "${INIT_PINCODE_PATH}" ]; then
		${REMOUNT_ROOT_WRITE_CMD} || return 1
		touch "${INIT_PINCODE_PATH}" || return  1
		${REMOUNT_ROOT_READ_ONLY_CMD} || return 1
	fi
	local PINCODE_CURRENT=`__pincode_current`
	if [ "${PINCODE_CURRENT}" != '' ]; then
		__pincode_delete "${PINCODE_CURRENT}" || return 128
	fi
	if ! __pincode_regist "${PINCODE}"; then
		[ "${KENSA}" = '' ] && return 128
		${REMOUNT_ROOT_WRITE_CMD}
		rm -f "${PINCODE_PATH}"
		${REMOUNT_ROOT_READ_ONLY_CMD}
		return 128
	fi
	${REMOUNT_ROOT_WRITE_CMD} || return 128
	if [ "${PINCODE}" != "${PINCODE_INIT}" ]; then
		echo -n "${PINCODE}" > "${PINCODE_INIT_PATH}" || return 128
	fi
	rm "${INIT_PINCODE_PATH}" || return  128
	${REMOUNT_ROOT_READ_ONLY_CMD} || return 1
	return 0
}

__pincode_init() {
	[ ! -f "${PINCODE_INIT_PATH}" ] && return 1
	cat "${PINCODE_INIT_PATH}" || return 1
	return 0
}

__pincode_current() {
	[ ! -f "${PINCODE_PATH}" ] && return 1
	cat "${PINCODE_PATH}" || return 1
	return 0
}

__pincode_new() {
	local TYPE="$1"
	local OPT='-t'
	[ "${TYPE}" = 'init' ] && OPT='-r'
	local UUID=`uuidgen ${OPT}`
	[ $? -ne 0 ] && return 1
	local PINCODE=`echo -n "${UUID}" | sed 's/\-//g' | tr 'a-z' 'A-Z'`
	[ $? -ne 0 ] && return 1
	echo -n "${PINCODE}"
	return 0
}

__pincode_delete() {
	local PINCODE="$1"
	[ "${PINCODE}" = '' ] && return 1

	local MACADDR=`cat "${MAC_PATH}"`

	#local POSTDATA="enc_type=2&pin_code=${PINCODE}&mac_address=${MACADDR}"
	local POSTDATA="enc_type=2&pin_code=${PINCODE}"
	local URI='http://localhost:15000/cs/InfoDeletion'
	local HTMLRESULT="`wget --timeout=30 --tries=1 --post-data="${POSTDATA}" "${URI}" -O -`"
	local RESULT=`echo -n "${HTMLRESULT}" | ${GETXMLRESULT}`
	[ "${RESULT}" = '00' -o "${RESULT}" = '01' ] || return 1

	${REMOUNT_ROOT_WRITE_CMD} || return 1
	rm -f "${PINCODE_PATH}" || return  1
	${REMOUNT_ROOT_READ_ONLY_CMD} || return 1

	return 0
}

__pincode_regist() {
	local PINCODE="$1"
	[ "${PINCODE}" = '' ] && return 1

	[ -f "${PINCODE_PATH}" ] && return 1
	${REMOUNT_ROOT_WRITE_CMD} || return 1
	echo -n "${PINCODE}" > "${PINCODE_PATH}" || return 1
	${REMOUNT_ROOT_READ_ONLY_CMD} || return 1

	#local MACADDR=`__localhost_macaddr`
	local MACADDR=`cat "${MAC_PATH}"`
	local AUTHORITY="0"
	local POSTDATA="enc_type=2&pin_code=${PINCODE}&mac_address=${MACADDR}&authority=${AUTHORITY}"
	local URI='http://localhost:15000/cs/InfoRegistration'
	local HTMLRESULT="`wget --timeout=30 --tries=1 --post-data="${POSTDATA}" "${URI}" -O -`"
	local RESULT=`echo -n "${HTMLRESULT}" | ${GETXMLRESULT}`
	[ "${RESULT}" = '00' ] || return 1

	return 0
}

__pincode_regist_current() {
	local PINCODE="$1"
	__pincode_regist "${PINCODE}"
}

if [ "${OPTION}" = 'new' ]; then
	__pincode_new || __exit 1
	__exit 0
fi
if [ "${OPTION}" = 'new_init' ]; then
	__pincode_new init || __exit 1
	__exit 0
fi
if [ "${OPTION}" = 'delete' -a "${PINCODE}" != '' ]; then
	__pincode_delete "${PINCODE}" || __exit 1
	__exit 0
fi
if [ "${OPTION}" = 'regist_init' -a "${PINCODE}" != '' ]; then
	__pincode_regist_init "${PINCODE}" 
	__exit $?
fi
if [ "${OPTION}" = 'regist_init_kensa' -a "${PINCODE}" != '' ]; then
	__pincode_regist_init "${PINCODE}" kensa
	__exit $?
fi

if [ -f "${INIT_PINCODE_PATH}" ]; then
	__pincode_regist_init || __exit 128
fi

case "${OPTION}" in
	test)
		__pincode_test || __exit 1
		;;
	regist_init)
		__pincode_regist_init "${PINCODE}"
		RESULT=$?
		[ ${RESULT} -ne 0 ] && __exit ${RESULT}
		;;
	init)
		__pincode_init || __exit 1
		;;
	current)
		__pincode_current || __exit 1
		;;
	new)
		__pincode_new || __exit 1
		;;
	new_init)
		__pincode_new init || __exit 1
		;;
	delete)
		PINCODE_CURRENT=`__pincode_current`
		__pincode_delete "${PINCODE_CURRENT}" || __exit 1
		;;
	regist)
		PINCODE_CURRENT=`__pincode_current`
		if [ "${PINCODE_CURRENT}" != '' ]; then
			__pincode_delete "${PINCODE_CURRENT}" || __exit 1
		fi
		__pincode_regist "${PINCODE}" || __exit 2
		;;
	*)
		__exit 1
		;;
esac

__exit 0
