#!/usr/bin/env python

import cgi
import cgitb
import sys
import os
import uuid
import hashlib
import json
import glob
import time
from ionas.request import (Entry, Request)
from fal_info import (FAL_USERS_PATH,)
from product_conf import (SUDO_CMD,)
from product_conf.reqinfo import (ReqTypes, Results)

cgitb.enable()


DEVCTRL_SESSION_DIR = "/var/tmp/devctrl/session"
GUEST_NAME = "guest"
MAX_SESSION_COUNT = 300
SESSION_VALID_TIME = 600

USER_PASSWORD_XPATH_TMPL = """userlist/user/name[text()="%s"]/../passwd"""

RESPONSE_BASE_DICT = {
        "response": "",
        "lang":  "ja",
        "result": 0,
    }

INVALID_RESPONSE_DICT = {
        "response": "",
        "lang":  "ja",
        "result": 1,
    }


class FileInfo(object):

    def __init__(self, path):

        self._path = path
        self._mtime = os.stat(path).st_mtime

    @property
    def path(self):

        return self._path

    @property
    def mtime(self):

        return self._mtime


def _request(fal_cmd, entry):
    response = Request(fal_cmd, filter_cmd=SUDO_CMD).request(entry)
    return response.get_entrylist().get_entry().get()


def _is_valid(user, hashed_password, salt):

    user_config = _request(FAL_USERS_PATH, Entry(ReqTypes.LOAD))
    raw_password = user_config.get_value(USER_PASSWORD_XPATH_TMPL % user)
    check_password = user + hashlib.sha256(raw_password).hexdigest() + salt
    
    if hashlib.sha256(check_password).hexdigest() == hashed_password:
        return True
    return False


def _try_delete_old_session_file():

    session_file_list = []
    for session_path in glob.glob(os.path.join(DEVCTRL_SESSION_DIR, "*")):
        session_file_list.append(FileInfo(session_path))

    session_file_list = sorted(session_file_list,
                                key=lambda file_info: file_info.mtime)
    if len(session_file_list) > MAX_SESSION_COUNT - 1:
        delete_count = len(session_file_list) - MAX_SESSION_COUNT + 1
        for file_info in session_file_list[0:delete_count]:
            os.unlink(file_info.path)
    
        session_file_list = session_file_list[delete_count:]

    for file_info in session_file_list:
        if time.time() - file_info.mtime > SESSION_VALID_TIME:
            os.unlink(file_info.path)


def auth(user, hashed_password, salt):

    if user:
        if not _is_valid(user, hashed_password, salt):
            return ""
    else:
        user=GUEST_NAME
    
    _try_delete_old_session_file()
    try:
        if not os.path.exists(DEVCTRL_SESSION_DIR):
            os.makedirs(DEVCTRL_SESSION_DIR)
        session_id = str(uuid.uuid1())
        session_path = os.path.join(DEVCTRL_SESSION_DIR, session_id)
        with open(session_path, "w") as session:
            session.write(user)
        return session_id
    except:
        return ""


#
# main
#

print "Content-Type: text/json\n"

try:
    request = json.loads(sys.stdin.read())
    response = dict(INVALID_RESPONSE_DICT)
    if request["request"] == "login":
        session_id = auth(request["name"],
                        request["password"], request.get("salt", ""))
        if session_id:
            response = dict(RESPONSE_BASE_DICT)
            response["session_id"] = session_id
        else:
            response = dict(INVALID_RESPONSE_DICT)

        response["response"] = "login"

    elif request["request"] == "logout":
        response = dict(RESPONSE_BASE_DICT)
        response["response"] = "logout"
        try:
            os.unlink(os.path.join(DEVCTRL_SESSION_DIR, request["session_id"]))
        except:
            pass

    print json.dumps(response, indent=1, ensure_ascii=False).encode("utf8")
except:
    import traceback as tb
    print json.dumps(INVALID_RESPONSE_DICT,
                    indent=1, ensure_ascii=False).encode("utf8")
